@@ -1,6 +1,6 @@
 """
 PyDetex
-https://github.com/ppizarror/pydetex
+https://github.com/ppizarror/PyDetex
 
 A python application that removes LaTeX code and transforms it to plain text for grammar checking or other things...
 """
@@ -14,8 +14,8 @@ __email__ = 'pablo@ppizarror.com'
 __keywords__ = 'latex detex parser gui'
 __license__ = 'MIT'
 __module_name__ = 'pydetex'
-__url__ = 'https://github.com/ppizarror/pydetex'
-__url_bug_tracker__ = 'https://github.com/ppizarror/pydetex'
-__url_documentation__ = 'https://github.com/ppizarror/pydetex'
-__url_source_code__ = 'https://github.com/ppizarror/pydetex'
+__url__ = 'https://github.com/ppizarror/PyDetex'
+__url_bug_tracker__ = 'https://github.com/ppizarror/PyDetex'
+__url_documentation__ = 'https://github.com/ppizarror/PyDetex'
+__url_source_code__ = 'https://github.com/ppizarror/PyDetex'
 __version__ = pydetex.version.ver
