@@ -89,10 +89,12 @@ class _LangManager(object):
                 'detected_lang_write': 'Write something to start recognizing the language',
                 'dictionary': 'Dictionary',
                 'dictionary_antonym': 'Antonym',
+                'dictionary_loading': 'Loading ...',
                 'dictionary_meaning': 'Meaning',
                 'dictionary_no_results': 'No results',
                 'dictionary_querying': 'Querying dictionary',
                 'dictionary_synonym': 'Synonym',
+                'dictionary_timeout': 'Error: Query timeout exceeded limit',
                 'dictionary_translation': 'Translation',
                 'dictionary_wikipedia': 'Wikipedia',
                 'dictionary_word': 'Word',
@@ -172,10 +174,12 @@ class _LangManager(object):
                 'detected_lang_write': 'Escribe algo para iniciar detección idioma',
                 'dictionary': 'Diccionario',
                 'dictionary_antonym': 'Antónimos',
+                'dictionary_loading': 'Cargando ...',
                 'dictionary_meaning': 'Definición',
                 'dictionary_no_results': 'Sin resultados',
                 'dictionary_querying': 'Consultando diccionario',
                 'dictionary_synonym': 'Sinónimos',
+                'dictionary_timeout': 'Error: El tiempo de consulta excedió el límite',
                 'dictionary_translation': 'Traducción',
                 'dictionary_wikipedia': 'Wikipedia',
                 'dictionary_word': 'Word',
